declare type Predicate<T> = (item: T, index: number, array: T[]) => unknown;
export declare class ArrayHelper {
    /**
     * Finds exactly one item in array. If more than one item has been found, an error will be thrown
     * @param array
     * @param predicate
     */
    findOne<T>(array: T[], predicate: Predicate<T>): T | undefined;
    /**
     * Finds one item in array. If more than one item has been found, first matching item will be returned.
     * @param array
     * @param predicate
     */
    findFirst<T>(array: T[], predicate: Predicate<T>): T | undefined;
    /**
     * True if at least one item is in array matching the predicate. If more than one item has been found method returns true.
     * @param array
     * @param predicate
     */
    contains<T>(array: T[], predicate: Predicate<T>): boolean;
    /**
     * True if exactly one item is in array matching the predicate. If more than one item has been found method returns false.
     * @param array
     * @param predicate
     */
    containsOne<T>(array: T[], predicate: Predicate<T>): boolean;
}
export default ArrayHelper;
